/*!
* rete-connection-path-plugin v2.0.3
* (c) 2024 Vitaliy Stoliarov
* Released under the MIT license.
* */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _typeof = require('@babel/runtime/helpers/typeof');
var _classCallCheck = require('@babel/runtime/helpers/classCallCheck');
var _createClass = require('@babel/runtime/helpers/createClass');
var _possibleConstructorReturn = require('@babel/runtime/helpers/possibleConstructorReturn');
var _getPrototypeOf = require('@babel/runtime/helpers/getPrototypeOf');
var _get = require('@babel/runtime/helpers/get');
var _inherits = require('@babel/runtime/helpers/inherits');
var _defineProperty = require('@babel/runtime/helpers/defineProperty');
var d3Shape = require('d3-shape');
var rete = require('rete');
var reteRenderUtils = require('rete-render-utils');
var _slicedToArray = require('@babel/runtime/helpers/slicedToArray');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var _typeof__default = /*#__PURE__*/_interopDefaultLegacy(_typeof);
var _classCallCheck__default = /*#__PURE__*/_interopDefaultLegacy(_classCallCheck);
var _createClass__default = /*#__PURE__*/_interopDefaultLegacy(_createClass);
var _possibleConstructorReturn__default = /*#__PURE__*/_interopDefaultLegacy(_possibleConstructorReturn);
var _getPrototypeOf__default = /*#__PURE__*/_interopDefaultLegacy(_getPrototypeOf);
var _get__default = /*#__PURE__*/_interopDefaultLegacy(_get);
var _inherits__default = /*#__PURE__*/_interopDefaultLegacy(_inherits);
var _defineProperty__default = /*#__PURE__*/_interopDefaultLegacy(_defineProperty);
var _slicedToArray__default = /*#__PURE__*/_interopDefaultLegacy(_slicedToArray);

var PathFactory = /*#__PURE__*/function () {
  function PathFactory(curve) {
    _classCallCheck__default["default"](this, PathFactory);
    this.curve = curve;
  }
  return _createClass__default["default"](PathFactory, [{
    key: "getData",
    value: function getData(points) {
      var getPath = d3Shape.line().x(function (d) {
        return d[0];
      }).y(function (d) {
        return d[1];
      }).curve(this.curve);
      return getPath(points.map(function (p) {
        return [p.x, p.y];
      }));
    }
  }]);
}();

/**
 * Built-in transformers, defining the auxiliary points for the path.
 * @module
 */

/**
 * Linear transformer. Returns the same points.
 * @returns Two points
 * @throws Error if number of points is not equal to 2
 */
var linear = function linear() {
  return function (points) {
    if (points.length !== 2) throw new Error('number of points should be equal to 2');
    var _points = _slicedToArray__default["default"](points, 2),
      start = _points[0],
      end = _points[1];
    return [start, end];
  };
};

/**
 * Classic transformer. Returns four points: start, auxiliary start, auxiliary end, end.
 * @param options Options
 * @param options.vertical If true, the auxiliary points will be placed vertically. Default: `false`
 * @param options.curvature The curvature of the path. Default: `0.3`
 * @returns Four points
 */
var classic = function classic(options) {
  var _options$vertical = options.vertical,
    vertical = _options$vertical === void 0 ? false : _options$vertical,
    _options$curvature = options.curvature,
    curvature = _options$curvature === void 0 ? 0.3 : _options$curvature;
  function add(a, b) {
    return {
      x: a.x + b.x,
      y: a.y + b.y
    };
  }
  return function (points) {
    if (points.length !== 2) throw new Error('number of points should be equal to 2');
    var _points2 = _slicedToArray__default["default"](points, 2),
      start = _points2[0],
      end = _points2[1];
    var xDistance = Math.abs(start.x - end.x);
    var yDistance = Math.abs(start.y - end.y);
    var crossDistance = vertical ? xDistance : yDistance;
    var alongDistance = vertical ? yDistance : xDistance;
    var offset = Math.max(crossDistance / 2, alongDistance) * curvature;
    var startOffset = vertical ? {
      x: 0,
      y: offset
    } : {
      y: 0,
      x: offset
    };
    var endOffset = vertical ? {
      x: 0,
      y: -offset
    } : {
      y: 0,
      x: -offset
    };
    return [start, add(start, startOffset), add(end, endOffset), end];
  };
};

var transformers = /*#__PURE__*/Object.freeze({
  __proto__: null,
  linear: linear,
  classic: classic
});

function getAngle(_ref, _ref2) {
  var x1 = _ref.x,
    y1 = _ref.y;
  var x2 = _ref2.x,
    y2 = _ref2.y;
  var dx = x1 - x2;
  var dy = y1 - y2;
  return 180 * Math.atan2(dy, dx) / Math.PI;
}
function getTransformAlong(path, offset) {
  var delta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;
  var needRotate = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : true;
  var length = path.getTotalLength() * delta;
  var p1 = path.getPointAtLength(length + offset);
  var p2 = path.getPointAtLength(length);
  var angle = 180 + (needRotate ? getAngle(p1, p2) : 0);
  return "translate(".concat(p1.x, ", ").concat(p1.y, ") rotate(").concat(angle, ")");
}

function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty__default["default"](e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _callSuper(t, o, e) { return o = _getPrototypeOf__default["default"](o), _possibleConstructorReturn__default["default"](t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf__default["default"](t).constructor) : o.apply(t, e)); }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _superPropGet(t, e, o, r) { var p = _get__default["default"](_getPrototypeOf__default["default"](1 & r ? t.prototype : t), e, o); return 2 & r && "function" == typeof p ? function (t) { return p.apply(o, t); } : p; }

/**
 * Connection path plugin props
 */

/**
 * Connection path plugin. Allows to customize connection path and arrow.
 * @listens connectionpath
 * @listens rendered
 */
var ConnectionPathPlugin = /*#__PURE__*/function (_Scope) {
  /**
   * @constructor
   * @param props Connection path plugin props
   */
  function ConnectionPathPlugin(props) {
    var _this;
    _classCallCheck__default["default"](this, ConnectionPathPlugin);
    _this = _callSuper(this, ConnectionPathPlugin, ['connection-path']);
    _defineProperty__default["default"](_this, "arrows", new Map());
    _defineProperty__default["default"](_this, "transforms", new Map());
    _this.props = props;
    return _this;
  }
  _inherits__default["default"](ConnectionPathPlugin, _Scope);
  return _createClass__default["default"](ConnectionPathPlugin, [{
    key: "setParent",
    value: function setParent(scope) {
      var _this2 = this;
      _superPropGet(ConnectionPathPlugin, "setParent", this, 3)([scope]);

      // eslint-disable-next-line max-statements,complexity
      scope.addPipe(function (context) {
        if (!context || _typeof__default["default"](context) !== 'object' || !('type' in context)) return context;
        if (context.type === 'connectionpath') {
          var _this2$props, _this2$props2;
          var _context$data = context.data,
            points = _context$data.points,
            payload = _context$data.payload;
          var curve = (_this2$props = _this2.props) !== null && _this2$props !== void 0 && _this2$props.curve ? _this2.props.curve(payload) : d3Shape.curveBundle.beta(0.9);
          var transformer = (_this2$props2 = _this2.props) !== null && _this2$props2 !== void 0 && _this2$props2.transformer ? _this2.props.transformer(payload) : classic({});
          var factory = new PathFactory(curve);
          var transformedPoints = transformer(points);
          var path = factory.getData(transformedPoints);
          var p = document.createElementNS('http://www.w3.org/2000/svg', 'path');
          p.setAttribute('d', path !== null && path !== void 0 ? path : reteRenderUtils.classicConnectionPath(transformedPoints, 0.3));
          _this2.transforms.set(payload.id, getTransformAlong(p, -15));
          _this2.updateArrow(payload);
          return path ? _objectSpread(_objectSpread({}, context), {}, {
            data: _objectSpread(_objectSpread({}, context.data), {}, {
              path: path
            })
          }) : context;
        }
        if (context.type === 'connectionremoved') {
          var id = context.data.id;
          _this2.arrows["delete"](id);
          _this2.transforms["delete"](id);
        }
        if (context.type === 'rendered' && context.data.type === 'connection') {
          var _this2$props3;
          if (!((_this2$props3 = _this2.props) !== null && _this2$props3 !== void 0 && _this2$props3.arrow) || context.data.type !== 'connection') return;
          var _payload = context.data.payload;
          var _id = _payload.id;
          var arrowData = _this2.getArrowData(_payload);
          if (_this2.arrows.has(_id) || !arrowData) return;
          var el = context.data.element;
          var svgEl = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
          var marker = document.createElementNS('http://www.w3.org/2000/svg', 'path');
          svgEl.setAttribute('style', 'position: absolute; overflow: visible !important; pointer-events: none');
          svgEl.appendChild(marker);
          el.appendChild(svgEl);
          marker.classList.add('marker');
          _this2.arrows.set(_id, {
            marker: marker
          });
          _this2.updateArrow(_payload);
        }
        return context;
      });
    }
  }, {
    key: "getArrowData",
    value: function getArrowData(c) {
      var _this$props;
      if (!((_this$props = this.props) !== null && _this$props !== void 0 && _this$props.arrow)) return null;
      var data = this.props.arrow(c);
      if (!data) return null;
      var _ref = data === true ? {} : data,
        _ref$color = _ref.color,
        color = _ref$color === void 0 ? 'steelblue' : _ref$color,
        _ref$marker = _ref.marker,
        marker = _ref$marker === void 0 ? 'M-5,-10 L-5,10 L20,0 z' : _ref$marker;
      return {
        color: color,
        marker: marker
      };
    }
  }, {
    key: "updateArrow",
    value: function updateArrow(c) {
      var _this$transforms$get;
      var data = this.arrows.get(c.id);
      var arrowData = this.getArrowData(c);
      if (!data || !arrowData) return;
      var color = arrowData.color,
        marker = arrowData.marker;
      data.marker.setAttribute('fill', color);
      data.marker.setAttribute('d', marker);
      data.marker.setAttribute('transform', (_this$transforms$get = this.transforms.get(c.id)) !== null && _this$transforms$get !== void 0 ? _this$transforms$get : '');
    }
  }]);
}(rete.Scope);

exports.ConnectionPathPlugin = ConnectionPathPlugin;
exports.Transformers = transformers;
//# sourceMappingURL=rete-connection-path-plugin.common.js.map
